/**********************************************************************************/
/*                                                                                */
/*        Copyright (C) 2005, WiNRADiO Communications, All Rights Reserved        */
/*                                                                                */
/**********************************************************************************/
#include <sys/ioctl.h>
#include <fcntl.h> 
#include <sys/soundcard.h>
#include "u_mainwindow.h"
#include "u_spectrum.h"

enum
{
    controlFrequency,
    controlCloseButton,
    controlAM,
    controlAMS,
    controlLSB,
    controlUSB,
    controlISB,
    controlDSB,
    controlCW,
    controlFM,
    controlIF1AGC_OFF,
    controlIF1AGC_Slow,
    controlIF1AGC_Medium,
    controlIF1AGC_Fast,
    
    controlSAGC_OFF,
    controlSAGC_Slow,
    controlSAGC_Medium,
    controlSAGC_Fast,
    
    controlIF1Gain,
    controlVolume,
    
    controlIFShift,
    controlIFBandwidth,
    
    timerID,
    timerID2
};

BEGIN_EVENT_TABLE( TMainWindow, wxFrame )
        EVT_FREQ_CHANGED(controlFrequency,TMainWindow::OnFreqChanged)
        EVT_BUTTON(controlCloseButton,TMainWindow::OnClose)
        EVT_BUTTON(controlAM,TMainWindow::OnMode)
        EVT_BUTTON(controlAMS,TMainWindow::OnMode)
        EVT_BUTTON(controlLSB,TMainWindow::OnMode)
        EVT_BUTTON(controlUSB,TMainWindow::OnMode)
        EVT_BUTTON(controlISB,TMainWindow::OnMode)
        EVT_BUTTON(controlDSB,TMainWindow::OnMode)
        EVT_BUTTON(controlCW,TMainWindow::OnMode)
        EVT_BUTTON(controlFM,TMainWindow::OnMode)
        EVT_BUTTON(controlIF1AGC_OFF,TMainWindow::OnIF1AGC)
        EVT_BUTTON(controlIF1AGC_Slow,TMainWindow::OnIF1AGC)
        EVT_BUTTON(controlIF1AGC_Medium,TMainWindow::OnIF1AGC)
        EVT_BUTTON(controlIF1AGC_Fast,TMainWindow::OnIF1AGC)
        
        EVT_BUTTON(controlSAGC_OFF,TMainWindow::OnSAGC)
        EVT_BUTTON(controlSAGC_Slow,TMainWindow::OnSAGC)
        EVT_BUTTON(controlSAGC_Medium,TMainWindow::OnSAGC)
        EVT_BUTTON(controlSAGC_Fast,TMainWindow::OnSAGC)
        
        //EVT_COMMAND_SCROLL_THUMBTRACK(controlIF1Gain,TMainWindow::OnIF1Gain)
        //EVT_COMMAND_SCROLL_THUMBTRACK(controlVolume,TMainWindow::OnVolume)
        EVT_TEXT_ENTER(controlIFShift,TMainWindow::OnIFShiftEnter)
        EVT_TEXT_ENTER(controlIFBandwidth,TMainWindow::OnIFBandwidthEnter)
        EVT_TIMER(timerID, TMainWindow::OnTimer)
        EVT_TIMER(timerID2,TMainWindow::OnTimer2)
END_EVENT_TABLE()

wxString AGCModes[]={"OFF","Slow","Medium","Fast"};

int IFShift[8]={0,0,0,0,0,0,0};
unsigned int IFBandwidth[8]={4000,4000,2300,2300,4600,4600,500,12000};


TMainWindow::TMainWindow(int hRadio,const wxString &Path):wxFrame(NULL,wxID_ANY,"",wxPoint(0,0),wxSize(500,405),
            wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxCLIP_CHILDREN|wxMINIMIZE_BOX),m_Timer(this,timerID),m_Timer2(this,timerID2)
{
 wxString Buffer;
 int i;
 wxFont FreqFont(24,wxFONTFAMILY_DEFAULT ,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_BOLD,FALSE);//wxFONTFLAG_ANTIALIASED|wxFONTWEIGHT_BOLDs
 wxFont ModeFont(10,wxFONTFAMILY_DEFAULT,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL,FALSE);
 wxFont AGCFont(18,wxFONTFAMILY_DEFAULT ,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_BOLD,FALSE);//wxFONTFLAG_ANTIALIASED|wxFONTWEIGHT_BOLDs
  
    m_Audio=open("/dev/dsp",O_RDWR);
    
    if(m_Audio>=0)
    {
        i=0;
        ioctl(m_Audio,SNDCTL_DSP_STEREO,&i);
        i=16000;
        ioctl(m_Audio,SNDCTL_DSP_SPEED,&i);
        i=AFMT_S16_LE;
        ioctl(m_Audio,SNDCTL_DSP_SETFMT,&i);
    }
 
    Centre();
    
    m_hRadio=hRadio;
    m_Path=Path;
    
    
    m_Info.Size=sizeof(m_Info);
    GetRadioInfo(hRadio,&m_Info);
    
    m_Serial=m_Info.SerNum;
    
    Buffer.Printf("WiNRADiO G313 Demo Application - %s [%s]",Path.c_str(),m_Info.SerNum);
    
    SetTitle(Buffer);
    
    wxStaticBox *g=new wxStaticBox(this,-1,"Frequency",wxPoint(10,10),wxSize(265,65));
    
    m_FreqEdit=new TFreqEdit(g,controlFrequency);
    m_FreqEdit->SetFont(FreqFont);
    m_FreqEdit->SetSize(wxSize(245,32));
    m_FreqEdit->SetPosition(wxPoint(10,20));
    
    g=new wxStaticBox(this,-1,"Mode",wxPoint(10,78),wxSize(265,58+25));
    
    wxString Modes[8]={"AM","AMS","LSB","USB","ISB","DSB","CW","FM"};
    int      ModeCtrls[8]={controlAM,controlAMS,controlLSB,controlUSB,controlISB,controlDSB,controlCW,controlFM};
    wxButton **Buttons[8];
    
    Buttons[0]=&m_AM;
    Buttons[1]=&m_AMS;
    Buttons[2]=&m_LSB;
    Buttons[3]=&m_USB;
    Buttons[4]=&m_ISB;
    Buttons[5]=&m_DSB;
    Buttons[6]=&m_CW;
    Buttons[7]=&m_FM;
    
    
    wxButton *Button;
    
    int k,l;
    
    for(int i=0;i<8;i++)
    {
        k=i>>2;
        l=i%4;
    
        Button=new wxButton(g,ModeCtrls[i],Modes[i]);//,wxPoint(20,10),wxSize(30,30));
        Button->SetPosition(wxPoint(l*41+10,20+26*k));
        Button->SetSize(wxSize(40,25));
        //Button->SetFont(ModeFont);
        
        *Buttons[i]=Button;
        m_Buttons[i]=Button;
    }
    
    m_ModeText=new wxStaticText(g,-1,"AM",wxPoint(195,33),wxSize(60,40));
    m_ModeText->SetFont(FreqFont);
    
    //Text=new wxStaticText(this,-1,"Frequency",wxPoint(10,10));
    
    m_CloseButton=new wxButton(this,controlCloseButton,"Close");//,wxPoint(415
    m_CloseButton->SetPosition(wxPoint(415,160+235-25));
    m_CloseButton->SetSize(wxSize(75,25));
    
    new wxStaticText(this,-1,"IF Bandwidth",wxPoint(415,235),wxSize(75,16));
    
    m_IFBandwidth=new wxTextCtrl(this,controlIFBandwidth,"",wxPoint(0,0),wxSize(0,0),wxTE_RIGHT|wxTE_PROCESS_ENTER);
    m_IFBandwidth->SetPosition(wxPoint(415,235+16));
    m_IFBandwidth->SetSize(wxSize(75,25));
    
    new wxStaticText(this,-1,"IF Shift",wxPoint(415,235+5+16+25),wxSize(75,16));
    
    m_IFShift=new wxTextCtrl(this,controlIFShift,"",wxPoint(0,0),wxSize(0,0),wxTE_RIGHT|wxTE_PROCESS_ENTER);
    m_IFShift->SetPosition(wxPoint(415,235+5+16+25+16));
    m_IFShift->SetSize(wxSize(75,25));    
    
    int IF1AGCCodes[]={controlIF1AGC_OFF,controlIF1AGC_Slow,controlIF1AGC_Medium,controlIF1AGC_Fast};
    
    g=new wxStaticBox(this,-1,"AGC",wxPoint(283,10),wxSize(100,151));
    
    for(int i=0;i<4;i++)
    {
        Button=new wxButton(g,IF1AGCCodes[i],AGCModes[i]);
        Button->SetPosition(wxPoint(10,i*25+20));
        Button->SetSize(wxSize(80,24));
    }
    
    m_IF1AGCText=new wxStaticText(g,-1,"Medium",wxPoint(10,125),wxSize(80,24),wxALIGN_CENTRE);
    
    m_IF1AGCText->SetFont(AGCFont);
    
    
    m_SoftAGCBox=new wxStaticBox(this,-1,"Software AGC",wxPoint(390,10),wxSize(100,151));
    
    int SAGCCodes[]={controlSAGC_OFF,controlSAGC_Slow,controlSAGC_Medium,controlSAGC_Fast};
    
    for(int i=0;i<4;i++)
    {
        Button=new wxButton(m_SoftAGCBox,SAGCCodes[i],AGCModes[i]);
        Button->SetPosition(wxPoint(10,i*25+20));
        Button->SetSize(wxSize(80,24));
        
        m_SoftAGCButtons[i]=Button;
    }
    
    m_SoftAGCText=new wxStaticText(m_SoftAGCBox,-1,"Medium",wxPoint(10,125),wxSize(80,24),wxALIGN_CENTRE);
    
    m_SoftAGCText->SetFont(AGCFont);    
    
    
    m_IF1GainBox=new wxStaticBox(this,-1,"IF Gain",wxPoint(10,164),wxSize(265,62));
    
    m_IF1Gain=new wxSlider(m_IF1GainBox,controlIF1Gain,0,0,100,wxDefaultPosition,wxDefaultSize,wxSL_AUTOTICKS|wxSL_HORIZONTAL|wxSL_RIGHT|wxSL_BOTH);
    m_IF1Gain->SetPosition(wxPoint(10,20));
    m_IF1Gain->SetSize(wxSize(205,30));
    m_IF1Gain->SetTickFreq(10,0);
    
    m_IF1GainText=new wxStaticText(m_IF1GainBox,-1,"",wxPoint(224,25),wxSize(30,20));
    m_IF1GainText->SetFont(AGCFont);
    
    
    this->Connect(controlIF1Gain,wxEVT_COMMAND_SLIDER_UPDATED,wxCommandEventHandler(TMainWindow::OnIF1Gain));
    this->Connect(controlVolume,wxEVT_COMMAND_SLIDER_UPDATED,wxCommandEventHandler(TMainWindow::OnVolume));
    
        
    
    g=new wxStaticBox(this,-1,"Volume",wxPoint(283,164),wxSize(207,62));
    
    m_Volume=new wxSlider(g,controlVolume,0,0,31,wxDefaultPosition,wxDefaultSize,wxSL_AUTOTICKS|wxSL_HORIZONTAL|wxSL_RIGHT|wxSL_BOTH);
    m_Volume->SetPosition(wxPoint(10,20));
    m_Volume->SetSize(wxSize(147,30));
    m_Volume->SetTickFreq(1,0);
    
    m_VolumeText=new wxStaticText(g,-1,"",wxPoint(166,25),wxSize(30,30));
    m_VolumeText->SetFont(AGCFont);
    
    m_Spectrum=new TSpectrum(this);
    
    m_Spectrum->SetSize(wxSize(398,160));
    m_Spectrum->SetPosition(wxPoint(10,235));
    
    
//     SetFrequency(hRadio,702000);
//     SetPower(hRadio,1);
//     SetVolume(hRadio,5);
    
    
    m_FreqEdit->SetMinMax(m_Info.MinFreq,m_Info.MaxFreq);
    
    unsigned int Freq;
    
    GetFrequency(hRadio,&Freq);
    
    m_FreqEdit->SetFreq(Freq,units_kHz);
    
    m_DontModifyFreq=false;
    
    
    LoadSettings();


    pthread_mutex_init(&m_Mutex,NULL);

    UpdateVolumeText();
    UpdateIF1GainText();
    

    m_Timer.Start(40);
}

TMainWindow::~TMainWindow(void)
{
    if(m_Audio>=0)
    {
        close(m_Audio);
    }


    SaveSettings();
    
    CloseDevice(m_hRadio);
    
    
    pthread_mutex_destroy(&m_Mutex);
    
    delete m_FreqEdit;
}


void TMainWindow::UpdateIF(void)
{
 char Buffer[64];
 int IFShift;
 unsigned int IFBandwidth;
     
    GetIFBandwidth(m_hRadio,&IFBandwidth);
     
    sprintf(Buffer,"%d",IFBandwidth);
    m_IFBandwidth->SetValue(Buffer);
   
    
    GetIFShift(m_hRadio,&IFShift);
    sprintf(Buffer,"%d",IFShift);
    m_IFShift->SetValue(Buffer);
}

void TMainWindow::OnClose(wxCommandEvent& event)
{
    Close();
}

void TMainWindow::OnFreqChanged(wxCommandEvent& event)
{
    if(SetFrequency(m_hRadio,m_FreqEdit->GetFreq()))
    {
        wxMessageBox("Unable to set frequency.","Error",wxOK|wxICON_STOP,this);
        event.Skip();
        //m_EditFreq->SetFreq(GetFrequency(m_hRadio),
    }
}

void TMainWindow::OnMode(wxCommandEvent& event)
{
 int Mode;
 
    switch(event.GetId())
    {
        case controlAM:
            Mode=RADIOMODE_AM;
            break;
        case controlAMS:
            Mode=RADIOMODE_SAM;
            break;
        case controlLSB:
            Mode=RADIOMODE_LSB;
            break;
        case controlUSB:
            Mode=RADIOMODE_USB;
            break;
        case controlISB:
            Mode=RADIOMODE_ISB;
            break;
        case controlDSB:
            Mode=RADIOMODE_DSB;
            break;
        case controlCW:
            Mode=RADIOMODE_CW;
            break;
        case controlFM:
            Mode=RADIOMODE_FMN;
            break;
    }
    
    if(SetMode(m_hRadio,Mode))
    {
        wxMessageBox("Unable to set mode.","Error",wxOK|wxICON_STOP,this);
    }
    
    
    m_Mode=Mode;
    
    SetIFShift(m_hRadio,IFShift[Mode]);
    SetIFBandwidth(m_hRadio,IFBandwidth[Mode]);
    
    UpdateIF();
    
    m_Spectrum->SetMode(Mode,IFShift[Mode],IFBandwidth[Mode]);
    
    UpdateModeText();
}

void TMainWindow::UpdateModeText(void)
{
 wxButton *Button;
 wxString Str;
 int Mode;
 
    GetMode(m_hRadio,&Mode);
    
    switch(Mode)
    {
        case RADIOMODE_AM:
            Button=m_AM;
            Str="AM";
            break;
        case RADIOMODE_SAM:
            Str="AMS";
            Button=m_AMS;
            break;
        case RADIOMODE_LSB:
            Str="LSB";
            Button=m_LSB;
            break;
        case RADIOMODE_USB:
            Str="USB";
            Button=m_USB;
            break;
        case RADIOMODE_ISB:
            Str="ISB";
            Button=m_ISB;
            break;
        case RADIOMODE_DSB:
            Str="DSB";
            Button=m_DSB;
            break;
        case RADIOMODE_CW:
            Str="CW";
            Button=m_CW;
            break;
        case RADIOMODE_FMN:
            Str="FM";
            Button=m_FM;
            break;
    }
    
    
    if(Mode==RADIOMODE_FMN)
    {
        for(int i=0;i<4;i++)
        {
            m_SoftAGCButtons[i]->Enable(false);
        }    
        m_SoftAGCText->Enable(false);
        m_SoftAGCBox->Enable(false);
    }
    else
    {
        for(int i=0;i<4;i++)
        {
            m_SoftAGCButtons[i]->Enable(true);
        }        
        m_SoftAGCText->Enable(true);
        m_SoftAGCBox->Enable(true);
    }
    
    m_ModeText->SetLabel(Str);
    
    /*
    for(int i=0;i<8;i++)
    {
        m_Buttons[i]->SetForegroundColour(0xffffffff);//m_DefaultColor);
    }
    
    Button->SetBackgroundColour(0xa0a0a0);*/
}


void TMainWindow::OnIF1AGC(wxCommandEvent& event)
{
 int AGC;
 wxString Str;
 
    switch(event.GetId())
    {
        case controlIF1AGC_OFF:
            AGC=0;            
            break;
        case controlIF1AGC_Slow:
            AGC=1;
            break;
        case controlIF1AGC_Medium:
            AGC=2;
            break;
        case controlIF1AGC_Fast:
            AGC=3;
            break;
    }
    
    
    
    if(SetAGC(m_hRadio,AGC))
    {
        wxMessageBox("Unable to set AGC.","Error",wxOK|wxICON_STOP,this);
    }


    UpdateIF1AGCText();
}

void TMainWindow::UpdateIF1AGCText(void)
{
 int AGC;
 
    GetAGC(m_hRadio,&AGC);
    
    if(AGC)
    {
        m_IF1GainText->Enable(false);
        m_IF1Gain->Enable(false);
        m_IF1GainBox->Enable(false);
    }
    else
    {
        m_IF1GainText->Enable(true);
        m_IF1Gain->Enable(true);
        m_IF1GainBox->Enable(true);
    }
    
    m_IF1AGCText->SetLabel(AGCModes[AGC]);
}

void TMainWindow::OnSAGC(wxCommandEvent& event)
{
 int AGC;
 wxString Str;
 
    
 
    switch(event.GetId())
    {
        case controlSAGC_OFF:
            AGC=0;            
            break;
        case controlSAGC_Slow:
            AGC=1;
            break;
        case controlSAGC_Medium:
            AGC=2;
            break;
        case controlSAGC_Fast:
            AGC=3;
            break;
    }
    
    if(SetSoftAGC(m_hRadio,AGC))
    {
        wxMessageBox("Unable to set software AGC.","Error",wxOK|wxICON_STOP,this);
    }
    
    UpdateSAGCText();
}

void TMainWindow::UpdateSAGCText(void)
{
 int AGC;
 
    GetSoftAGC(m_hRadio,&AGC);
 
    m_SoftAGCText->SetLabel(AGCModes[AGC]);    
}

void TMainWindow::OnIF1Gain(wxCommandEvent &event)
{
 int Gain;
 wxString Str;
 
    Gain=m_IF1Gain->GetValue();
        
    if(SetIFGain(m_hRadio,Gain))
    {
        wxMessageBox("Unable to set IF gain.","Error",wxOK|wxICON_STOP,this);
    }
    UpdateIF1GainText();
}

void TMainWindow::UpdateIF1GainText(void)
{
 wxString Str;
 unsigned int Gain;
 
    GetIFGain(m_hRadio,&Gain);
 
    Str.Printf("%d",Gain);
    m_IF1GainText->SetLabel(Str); 
}

void TMainWindow::OnVolume(wxCommandEvent &event)
{
 int Volume;
 wxString Str;
        
        Volume=m_Volume->GetValue();
        
        if(SetVolume(m_hRadio,Volume))
        {
            wxMessageBox("Unable to set volume.","Error",wxOK|wxICON_STOP);
        }        
        
        UpdateVolumeText();
}

void TMainWindow::UpdateVolumeText(void)
{
 wxString Str;
 unsigned int Volume; 
 
    GetVolume(m_hRadio,&Volume);
 
    Str.Printf("%d",Volume);
    m_VolumeText->SetLabel(Str);
}

typedef struct
{
    unsigned int   Frequency;
    int   Mode;
    int   IF1AGC;
    unsigned int   IF1Gain;
    int   SAGC;
    unsigned int   Volume;
    int   Units;
} WRG313_SETTINGS;

void TMainWindow::LoadSettings(void)
{
 FILE *F;
 wxString FilePath;
 WRG313_SETTINGS Settings,Settings2;
 char *Home;
 char Buff[2048];
 
    SetVolume(m_hRadio,0);
 
    Settings.Frequency=m_Info.MinFreq;
    Settings.Mode=RADIOMODE_AM;
    Settings.Volume=10;
    Settings.IF1AGC=2;
    Settings.IF1Gain=80;
    Settings.SAGC=0;
    Settings.Units=units_kHz;
 
    getcwd(Buff,2048);
    
    Home=getenv("HOME");
    
    chdir(Home);
 
    FilePath.Printf("./.WiNRADiO/G313/demo-application/%s.cfg",m_Serial.c_str());
 
    F=fopen(FilePath.c_str(),"rb");
    
    if(F)
    {
        if(fread(&Settings2,1,sizeof(Settings2),F)==sizeof(Settings2))
        {
            Settings=Settings2;
        }
        fclose(F);
    }
    
    chdir(Buff);
    
    
    m_IF1Gain->SetValue(Settings.IF1Gain);
    m_Volume->SetValue(Settings.Volume);
    
    if(Settings.Units>3 || Settings.Units<0)
    {
        Settings.Units=1;
    }
    
    if(Settings.Mode<0 || Settings.Mode>7)
    {
        Settings.Mode=RADIOMODE_AM;
    }
    
    m_FreqEdit->SetFreq(Settings.Frequency,(TFreqUnits)Settings.Units);
    
    SetAGC(m_hRadio,Settings.IF1AGC);
    SetIFGain(m_hRadio,Settings.IF1Gain);
    SetSoftAGC(m_hRadio,Settings.SAGC);
    
    m_Mode=Settings.Mode;
    SetMode(m_hRadio,Settings.Mode);
    SetFrequency(m_hRadio,Settings.Frequency);
    
    GetIFShift(m_hRadio,&IFShift[Settings.Mode]);
    GetIFBandwidth(m_hRadio,&IFBandwidth[Settings.Mode]);
    
    UpdateIF();
    
    m_Spectrum->SetMode(Settings.Mode,IFShift[Settings.Mode],IFBandwidth[Settings.Mode]);
    
    UpdateVolumeText();
    UpdateIF1GainText();
    UpdateIF1AGCText();
    UpdateSAGCText();
    UpdateModeText();
    
    SetPower(m_hRadio,1);
    
    usleep(100000);
    
    SetVolume(m_hRadio,Settings.Volume);
}



void TMainWindow::SaveSettings(void)
{
 char Buffer[2048];
 char *Home;
 WRG313_SETTINGS Settings;
 FILE *F;
 wxString FileName;
 
    Home=getenv("HOME");
    
    getcwd(Buffer,2048);
    
    if(chdir(Home)<0)
    {        
        return;
    }
    
    umask(0x12); //022
    
    mkdir(".WiNRADiO",0x1E4 /*0744*/);
    
    if(chdir("./.WiNRADiO"))
    {
        return;
    }
    
    mkdir("G313",0x1E4);
    
    if(chdir("./G313"))
    {
        return;
    }
    
    mkdir("demo-application",0x1E4);
    
    if(chdir("./demo-application"))
    {
        return;
    }
    
    GetFrequency(m_hRadio,&Settings.Frequency);
    GetMode(m_hRadio,&Settings.Mode);
    GetVolume(m_hRadio,&Settings.Volume);
    GetAGC(m_hRadio,&Settings.IF1AGC);
    GetIFGain(m_hRadio,&Settings.IF1Gain);
    GetSoftAGC(m_hRadio,&Settings.SAGC);
    Settings.Units=(int)m_FreqEdit->GetUnits();
    
    FileName.Printf("%s.cfg",m_Serial.c_str());
    
    F=fopen(FileName.c_str(),"wb");
    
    if(F)
    {
        fwrite(&Settings,1,sizeof(Settings),F);
        fclose(F);
    }
    
    chdir(Buffer);
}



void TMainWindow::OnTimer(wxTimerEvent& event)
{
    pthread_mutex_lock(&m_Mutex);
    
    if(m_SpectrumChanged)
    {
        m_Spectrum->SetSpectrum(m_SpectrumData);
        m_SpectrumChanged=false;
    }
    
    pthread_mutex_unlock(&m_Mutex); 
}

void TMainWindow::SpectrumCallback(float *Buffer,int Count)
{
    pthread_mutex_lock(&m_Mutex);
    
    m_SpectrumChanged=true;
    memcpy(m_SpectrumData,Buffer,1024*sizeof(float)); //Count is always 854
    
    pthread_mutex_unlock(&m_Mutex);
}

void TMainWindow::AudioCallback(short *Buffer,int Count)
{
    if(m_Audio>=0)
    {
        write(m_Audio,Buffer,Count*2);
    }    
}

void TMainWindow::SpectrumCallback(float *Buffer,int Count,void* UserData)
{
    ((TMainWindow*)UserData)->SpectrumCallback(Buffer,Count);
}

void TMainWindow::AudioCallback(short *Buffer,int Count,void *UserData)
{
    ((TMainWindow*)UserData)->AudioCallback(Buffer,Count);
}

void TMainWindow::OnIFShiftChanged(int NewIFShift)
{
    SetIFShift(m_hRadio,NewIFShift);
    IFShift[m_Mode]=NewIFShift;
    
    UpdateIF();
}


void TMainWindow::OnTimer2(wxTimerEvent& event)
{
    SetIFBandwidth(m_hRadio,IFBandwidth[m_Mode]);
    UpdateIF();
    m_Timer2.Stop();
}
void TMainWindow::OnIFBandwidthChanged(int NewIFBandwidth)
{
    m_Timer2.Stop();
    m_Timer2.Start(10,true);

    IFBandwidth[m_Mode]=NewIFBandwidth;
    
    UpdateIF();
}

void TMainWindow::OnIFShiftEnter(wxCommandEvent &event)
{
 wxString Value;
 int IFShift;
 char *end;
 char Buffer[16];
 unsigned int Bandwidth;
 int Min,Max;
 
    Value=m_IFShift->GetValue();
    IFShift=strtol(Value.c_str(),&end,10);
    
    if(*end)
    {
        //GetIFShift(m_hRadio,&IFShift);
        //sprintf(Buffer,"%d",IFShift);
        //m_IFShift->SetValue(Buffer);
        return;
    }
    
    GetIFBandwidth(m_hRadio,&Bandwidth);
    
    switch(m_Mode)
    {
        case RADIOMODE_LSB:
            Max=7500;
            Min=Bandwidth-7500;
            break;
        case RADIOMODE_USB:
            Min=-7500;
            Max=7500-Bandwidth;
            break;
        default:
            Min=Bandwidth/2-7500;
            Max=7500-Bandwidth/2;
            break;
    }
    
    if(IFShift<Min)
    {
        IFShift=Min;
    }
    else if(IFShift>Max)
    {
        IFShift=Max;
    }
    
    SetIFShift(m_hRadio,IFShift);
    m_Spectrum->SetMode(m_Mode,IFShift,Bandwidth);
    ::IFShift[m_Mode]=IFShift;
    
    sprintf(Buffer,"%d",IFShift);
    m_IFShift->SetValue(Buffer);    
}

void TMainWindow::OnIFBandwidthEnter(wxCommandEvent &event)
{
 int IFShift;
 int Bandwidth;
 wxString Value;
 char *end;
 int Max;
 char Buffer[32];
 
    Value=m_IFBandwidth->GetValue();
    
    Bandwidth=strtol(Value.c_str(),&end,10);
    
    
    if(*end || Bandwidth<0)
    {
        return;
    }
    
    GetIFShift(m_hRadio,&IFShift);
    
    switch(m_Mode)
    {
        case RADIOMODE_USB:
            Max=7500-IFShift;
            break;
        case RADIOMODE_LSB:
            Max=7500+IFShift;
            break;
        default:
            if(IFShift<=0)
            {
                Max=2*(IFShift+7500);
            }
            else
            {
                Max=2*(7500-IFShift);
            }
            break;
    }
    
    if(Bandwidth>Max)
    {
        Bandwidth=Max;
    }
    
    if(Bandwidth<100)
    {
        Bandwidth=100;
    }
    
    m_Spectrum->SetMode(m_Mode,IFShift,Bandwidth);
    SetIFBandwidth(m_hRadio,Bandwidth);
    ::IFBandwidth[m_Mode]=Bandwidth;
    
    sprintf(Buffer,"%d",Bandwidth);
    m_IFBandwidth->SetValue(Buffer);    
        
}
