/**********************************************************************************/
/*                                                                                */
/*        Copyright (C) 2005, WiNRADiO Communications, All Rights Reserved        */
/*                                                                                */
/**********************************************************************************/

#include "u_spectrum.h"
#include "wrg313api.h"
#include "u_mainwindow.h"

#define MOUSE_MOVE        1
#define MOUSE_SIZE_LEFT   2
#define MOUSE_SIZE_RIGHT  3

BEGIN_EVENT_TABLE(TSpectrum, wxWindow)
    EVT_PAINT       (TSpectrum::OnPaint)
    EVT_SIZE        (TSpectrum::OnSize)
    EVT_MOTION      (TSpectrum::OnMouseMove)
    EVT_LEFT_DOWN   (TSpectrum::OnLeftMouseDown)
    EVT_LEFT_UP     (TSpectrum::OnLeftMouseUp)    
END_EVENT_TABLE()    

TSpectrum::TSpectrum(TMainWindow *Parent):wxWindow((wxWindow*)Parent,-1,wxDefaultPosition,wxDefaultSize,wxSUNKEN_BORDER,""),
  m_Font(9,wxFONTFAMILY_MODERN,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL),
  m_Cursor_Size(wxCURSOR_SIZEWE),m_Cursor_Move(wxCURSOR_SIZENESW),
  m_Text_Color(0,0xFF,0),
  m_Brush_Background(wxColour(0,0,0)),m_Brush_Background_Active(wxColour(0x60,0x60,0x60)),
  m_Pen_Spectrum(wxColour(0xFF,0xFF,0xFF)),m_Pen_Spectrum_Active(wxColour(0xFF,0xFF,0)),
  m_Pen_Grid(wxColour(0,0x90,0)),m_Pen_Grid_Light(wxColour(0,0xFF,0)),
  m_Pen_Grid_Active(wxColour(0x30,0x90,0x30)),m_Pen_Grid_Light_Active(wxColour(0x30,0xC0,0x30)),
  m_Pen_Axis(wxColour(0,0xFF,0),2),
  m_Pen_Frame_Active(wxColour(0xFF,0,0)),
  m_Pen_IFCenter(wxColour(0xA0,0,0)),
  m_Pen_MinMax(wxColour(0x50,0x50,0)),
  ValidData(false)
{
    m_SpectrumLineY=NULL;
    m_Points=NULL;
    m_IFShift=0;
    m_IFBandwidth=4000;
    m_Mode=RADIOMODE_AM;
    m_ActiveCursor=wxCURSOR_ARROW;
    m_LButtonDown=0;
    m_MaxIFBandwidth=15000;
    
    m_MainWindow=Parent;
}

TSpectrum::~TSpectrum(void)
{
    if(m_SpectrumLineY)
    {
        delete[] m_SpectrumLineY;
    }
    
    if(m_Points)
    {
        delete[] m_Points;
    }
}

void TSpectrum::SetMode(int Mode,int IFShift,int IFBandwidth)
{
    m_Mode=Mode;

    m_IFShift=IFShift;
    m_IFBandwidth=IFBandwidth;

    CalculateMaxIFBandwidth();
    
    if(m_IFBandwidth>m_MaxIFBandwidth)
    {   
        m_IFBandwidth=m_MaxIFBandwidth;
    }
    
    CalculateIFBandwidthRect();
    Refresh(false);
}

void TSpectrum::CalculateMaxIFBandwidth(void)
{
    switch(m_Mode)
    {
        case RADIOMODE_USB:
            m_MaxIFBandwidth=7500-m_IFShift;
            break;
        case RADIOMODE_LSB:
            m_MaxIFBandwidth=m_IFShift+7500;
            break;
        default:
            if(m_IFShift<=0)
            {
                m_MaxIFBandwidth=2*(m_IFShift+7500);
            }
            else
            {
                m_MaxIFBandwidth=2*(7500-m_IFShift);
            }
            break;
    }
    
    if(m_MaxIFBandwidth>15000)
    {
        m_MaxIFBandwidth=15000;
    }
}

void TSpectrum::ChangeCursor(int Cursor)
{
    if(m_ActiveCursor!=Cursor)
    {
        m_ActiveCursor=Cursor;
        switch(Cursor)
        {
            case wxCURSOR_SIZEWE:
                SetCursor(m_Cursor_Size);
                break;
            case wxCURSOR_SIZENESW:
                SetCursor(m_Cursor_Move);
                break;
            default:
                SetCursor(*wxSTANDARD_CURSOR);
                break;
        }
    }
}

void TSpectrum::SetSpectrum(float *Data)
{
    memcpy(m_Spectrum,Data+85,854*sizeof(float));
    ValidData=true;
    Refresh(false);
}

void TSpectrum::OnSize(wxSizeEvent &event)
{
    CalculateParameters();
   
    event.Skip();
}

void TSpectrum::CorrectIFShift(void)
{
 int b2;
    switch(m_Mode)
    {
        case RADIOMODE_LSB:
            if(m_IFShift-m_IFBandwidth<-7500)
            {
                m_IFShift=-7500+m_IFBandwidth;
            }
            else if(m_IFShift>7500)
            {
                m_IFShift=7500;
            }
            break;
        case RADIOMODE_USB:
            if(m_IFShift<-7500)
            {
                m_IFShift=-7500;
            }
            else if(m_IFShift+m_IFBandwidth>7500)
            {
                m_IFShift=7500-m_IFBandwidth;
            }
            break;
        default:
            b2=m_IFBandwidth/2;
            if(m_IFShift-b2<-7500)
            {
                m_IFShift=-7500+b2;
            }
            else if(m_IFShift+b2>7500)
            {
                m_IFShift=7500-b2;
            }
            break;
    }
}

void TSpectrum::OnMouseMove(wxMouseEvent &event)
{
 int dx;
 double df;
 int OldIFShift;
 int OldIFBandwidth;
 double k;
 
    switch(m_LButtonDown)
    {
        case MOUSE_SIZE_LEFT:
        case MOUSE_SIZE_RIGHT:
            OldIFShift=m_IFShift;
            OldIFBandwidth=m_IFBandwidth;
            if(m_LButtonDown==MOUSE_SIZE_LEFT)
            {
                dx=m_LButtonDownX-event.m_x;
            }
            else
            {
                dx=event.m_x-m_LButtonDownX;    
            }
            
            if(m_Mode!=RADIOMODE_LSB && m_Mode!=RADIOMODE_USB)
            {
                dx*=2;
                k=2.0;
            }
            else
            {
                k=1.0;
            }
            
            df=dx/m_Frequency_kx;            

            m_IFBandwidth=(int)((double)m_LButtonDownIFBandwidth+df);
                    
            if(m_IFBandwidth>m_MaxIFBandwidth)
            {
                m_IFBandwidth=m_MaxIFBandwidth;
            }
            else if(m_IFBandwidth<100)
            {
                m_IFBandwidth=100;
            }
            
        
            //CorrectIFShift();
        
        
            if(OldIFShift!=m_IFShift || OldIFBandwidth!=m_IFBandwidth)
            {
                if(OldIFShift!=m_IFShift)
                {
                    m_MainWindow->OnIFShiftChanged(m_IFShift);
                }
                
                if(OldIFBandwidth!=m_IFBandwidth)
                {
                    m_MainWindow->OnIFBandwidthChanged(m_IFBandwidth);
                }
                
                CalculateIFBandwidthRect();
                Refresh(false);                          
            }
            
            
            break;
        case MOUSE_MOVE:
            OldIFShift=m_IFShift;
            dx=m_LButtonDownX-event.m_x;
            df=dx/m_Frequency_kx;
            m_IFShift=(int)((double)m_LButtonDownIFShift-df);

            CorrectIFShift();
            
            CalculateMaxIFBandwidth();
            
            if(OldIFShift!=m_IFShift)
            {
                m_MainWindow->OnIFShiftChanged(m_IFShift);
                CalculateIFBandwidthRect();
                Refresh(false);                
            }
            break;
        default:
            switch(GetMousePos(event.m_x,event.m_y))
            {
                case MOUSE_SIZE_LEFT:
                case MOUSE_SIZE_RIGHT:
                    ChangeCursor(wxCURSOR_SIZEWE);
                    break;            
                case MOUSE_MOVE:
                    ChangeCursor(wxCURSOR_SIZENESW);
                    break;            
                default:
                    ChangeCursor(wxCURSOR_ARROW);
                    break;
            }
            break;
    }
}


int TSpectrum::GetMousePos(int x,int y)
{
 int x1,y1,x2,y2;
 
    x1=m_IFBandwidthX1;
    y1=m_TopOffset;
    x2=m_IFBandwidthX2;
    y2=m_SpectrumHeight+y1;
    
    if(x>=x1 && x<=x2 && 
       y>=y1 && y<=y2)
    {
        if(x-x1<=2 && m_Mode!=RADIOMODE_USB)
        {
            return MOUSE_SIZE_LEFT;
        }
        else if(x2-x<=2 && m_Mode!=RADIOMODE_LSB)
        {
            
            return MOUSE_SIZE_RIGHT;
        }
        else
        {
            return MOUSE_MOVE;   
        }
    }
    else
    {
        return 0;
    }
}

void TSpectrum::OnLeftMouseDown(wxMouseEvent& event)
{
    m_LButtonDown=GetMousePos(event.m_x,event.m_y);
    if(m_LButtonDown)
    {
        m_LButtonDownIFShift=m_IFShift;
        m_LButtonDownIFBandwidth=m_IFBandwidth;
        m_LButtonDownX=event.m_x;
    }
}

void TSpectrum::OnLeftMouseUp(wxMouseEvent& event)
{
    m_LButtonDown=0;
}

void TSpectrum::CalculateParameters(void)
{
 int Width,Height;
 int i;
 double frequency;
 double magnitude;
 wxBitmap Bitmap(100,100);
 wxMemoryDC DC;
 int TextWidth,TextHeight;
    
    
    PrepareDC(DC);
    
    DC.SetMapMode(wxMM_TEXT);
    DC.SelectObject(Bitmap);
    DC.SetFont(m_Font);
    
    DC.GetTextExtent("0000dB",&TextWidth,&TextHeight);
 
    GetClientSize(&Width,&Height);
    
    if(m_SpectrumLineY)
    {
        delete[] m_SpectrumLineY;
    }
    
    m_SpectrumLineY=new int[Width];
    
    
    if(m_Points)
    {
        delete[] m_Points;
    }
    
    m_Points=new wxPoint[Width];
    
    m_LeftOffset=4+TextWidth+4;
    m_TopOffset=4+TextHeight/2;
    m_SpectrumWidth=Width-m_LeftOffset-TextWidth/2;
    m_SpectrumHeight=Height-m_TopOffset-2-TextHeight-TextHeight/2-2;
 
    m_Spectrum_kx=(double)m_SpectrumWidth/854.0;
    m_Spectrum_ky=(double)m_SpectrumHeight/100.0; //0dB ro -100dB
    

    
    m_FramePoints[0]=wxPoint(m_LeftOffset,m_TopOffset);

    m_FramePoints[1]=wxPoint(m_LeftOffset,m_TopOffset+m_SpectrumHeight);

    m_FramePoints[2]=wxPoint(m_LeftOffset+m_SpectrumWidth,m_TopOffset+m_SpectrumHeight);

    
    m_Frequency_kx=(double)m_SpectrumWidth/20015.625;
    
    for(frequency=0.0,i=0;i<=20;i++,frequency+=1000.0)
    {
        m_GridX[i]=(int)((double)m_LeftOffset+frequency*m_Frequency_kx);
    }
    
     
    for(magnitude=0.0,i=0;i<=10;i++,magnitude+=10.0)
    {
        m_GridY[i]=(int)((double)m_TopOffset+magnitude*m_Spectrum_ky);
    }
    

    m_MinFrequencyX=(int)((double)m_LeftOffset+(10000.0-7500.0)*m_Frequency_kx);
    m_MaxFrequencyX=(int)((double)m_LeftOffset+(10000.0+7500.0)*m_Frequency_kx);    

    CalculateIFBandwidthRect();
}
 
void TSpectrum::CalculateIFBandwidthRect(void)
{ 
 double Left,Width;
 
    //m_IFBandwidthRect=wxRect    
    
    Width=(double)m_IFBandwidth*m_Frequency_kx;
    
    if(Width<3)
    {
        Width=3;
    }
    
    switch(m_Mode)
    {
        case RADIOMODE_LSB:
            Left=(m_IFShift+10000.0)*m_Frequency_kx;
            Left-=Width;
            m_IFBandwidthX2=m_IFShiftX=(int)(Left+Width);
            m_IFBandwidthX1=(int)Left;
            break;
        case RADIOMODE_USB:
            Left=((double)m_IFShift+10000.0)*m_Frequency_kx;
            m_IFBandwidthX1=m_IFShiftX=(int)Left;
            m_IFBandwidthX2=(int)(Left+Width);
            break;
        default:
            Left=((double)m_IFShift+10000.0)*m_Frequency_kx;
            Left-=Width/2.0;
            m_IFBandwidthX1=(int)Left;
            m_IFBandwidthX2=(int)(Left+Width);
            m_IFShiftX=(int)(Left+Width/2.0);
            break;
    }
    
    m_IFBandwidthX1+=m_LeftOffset;
    m_IFBandwidthX2+=m_LeftOffset;
    m_IFShiftX+=m_LeftOffset;    
}

void TSpectrum::OnPaint(wxPaintEvent &event)
{
 wxPaintDC WindowDC(this);
 wxBitmap *Bitmap;
 wxMemoryDC DC;
 int Width,Height;
 int AWidth,AHeight;
 double fy;
 int x,y;
 float *Spectrum;
 int *SpectrumLineY;
 int i;
 int magnitude;
 char Num[16],Buffer[16];
 int TextWidth,TextHeight;
 int frequency;
    
    if(m_SpectrumLineY==NULL)
    {
        return;
    }
    
    GetClientSize(&Width,&Height);
    
    Bitmap=new wxBitmap(Width,Height);
    
    PrepareDC(DC);

    DC.SetMapMode(wxMM_TEXT);

    //m_owner->PrepareDC(DC);
    
    DC.SelectObject(*Bitmap);
    
    
    DC.SetBackground(m_Brush_Background);
    DC.Clear();
    
    
    if(ValidData)
    {
        memset(m_SpectrumLineY,0,Width*sizeof(int));
    
        //the case when the spectrum width is greater than size if spectrum(number of items in the spectrum array)
        //is not implemented here
        Spectrum=m_Spectrum;
        SpectrumLineY=m_SpectrumLineY;
        for(i=0;i<854;i++)
        {
            x=(int)((double)i*m_Spectrum_kx);
            fy=*Spectrum++;
            if(fy<-100)
            {
                fy=-100;
            }
            else if(fy>0)
            {
                fy=0;
            }
            
            fy+=100; 
            fy*=m_Spectrum_ky;
            
            if(m_SpectrumLineY[x]<fy)
            {
                m_SpectrumLineY[x]=(int)fy;
            }
        }
    
        for(i=0;i<m_SpectrumWidth;i++)
        {
            m_Points[i].x=i;
            m_Points[i].y=m_SpectrumHeight-m_SpectrumLineY[i];
        }
    }


    DC.SetPen(m_Pen_MinMax);
    DC.DrawLine(m_MinFrequencyX,m_TopOffset,m_MinFrequencyX,m_TopOffset+m_SpectrumHeight);
    DC.DrawLine(m_MaxFrequencyX,m_TopOffset,m_MaxFrequencyX,m_TopOffset+m_SpectrumHeight);

    DC.SetPen(m_Pen_Grid);
    DC.SetFont(m_Font);
    DC.SetTextForeground(m_Text_Color);
    

    DC.GetTextExtent("0000",&TextWidth,&TextHeight);
    TextHeight/=2;
      
    for(i=0,magnitude=0;i<=10;i++,magnitude-=10)
    {
        DC.DrawLine(m_LeftOffset,m_GridY[i],m_LeftOffset+m_SpectrumWidth,m_GridY[i]);
        
        if(magnitude%20==0)
        {
            sprintf(Num,"%d""dB",magnitude);
            sprintf(Buffer,"%6s",Num);
            DC.DrawText(Buffer,4,m_GridY[i]-TextHeight);
        }
    }
    
    DC.GetTextExtent("0000",&TextWidth,&TextHeight);
         
    y=m_TopOffset+m_SpectrumHeight+2+TextHeight/2;
    
    for(frequency=-10,i=0;i<=20;i++,frequency++)
    {
        if(frequency%5 == 0)
        {
            if(frequency)
            {
                sprintf(Num,"%+dkHz",frequency);
                sprintf(Buffer,"%s",Num);
            }
            else
            {
                Buffer[0]='0';
                Buffer[1]='\0';
            }
            DC.GetTextExtent(Buffer,&TextWidth,&TextHeight);
            DC.DrawText(Buffer,m_GridX[i]-TextWidth/2,y);
            
            DC.SetPen(m_Pen_Grid_Light);
            DC.DrawLine(m_GridX[i],m_TopOffset,m_GridX[i],m_TopOffset+m_SpectrumHeight);        
            DC.SetPen(m_Pen_Grid);
        }
        else
        {
            DC.DrawLine(m_GridX[i],m_TopOffset,m_GridX[i],m_TopOffset+m_SpectrumHeight);        
        }
    }
    

    //draw IFshift,IFbandwidth rect
    
    DC.SetBrush(m_Brush_Background_Active);
    DC.SetPen(m_Pen_Frame_Active);
    
    AWidth=m_IFBandwidthX2-m_IFBandwidthX1+1;
    AHeight=m_SpectrumHeight-1; 
    
    DC.DrawRectangle(m_IFBandwidthX1,m_TopOffset,AWidth,AHeight);


    if(ValidData)
    {
        //draw the spectrum lines
        DC.SetPen(m_Pen_Spectrum);
        DC.DrawLines(m_IFBandwidthX1-m_LeftOffset,m_Points,m_LeftOffset,m_TopOffset);
        x=m_IFBandwidthX2-m_LeftOffset;
        DC.DrawLines(m_SpectrumWidth-x,m_Points+x,m_LeftOffset,m_TopOffset);
    }
    
    DC.SetPen(m_Pen_Axis);
    DC.DrawLines(3,m_FramePoints,0,0);
    


    DC.SetClippingRegion(m_IFBandwidthX1+1,m_TopOffset+1,AWidth-2,AHeight-3);
    DC.SetPen(m_Pen_Grid_Active);
    for(i=0,magnitude=0;i<=10;i++,magnitude-=10)
    {
        DC.DrawLine(m_LeftOffset,m_GridY[i],m_LeftOffset+m_SpectrumWidth,m_GridY[i]);
    }
    
         
    y=m_TopOffset+m_SpectrumHeight+2+TextHeight/2;
    
    
    for(frequency=-10,i=0;i<=20;i++,frequency++)
    {
        
        if(frequency%5 == 0)
        {            
            DC.SetPen(m_Pen_Grid_Light_Active);
            DC.DrawLine(m_GridX[i],m_TopOffset,m_GridX[i],m_TopOffset+m_SpectrumHeight);        
            DC.SetPen(m_Pen_Grid_Active);
        }
        else
        {
            DC.DrawLine(m_GridX[i],m_TopOffset,m_GridX[i],m_TopOffset+m_SpectrumHeight);        
        }
    }
    
    DC.DestroyClippingRegion();

    DC.SetPen(m_Pen_IFCenter);
    DC.DrawLine(m_IFShiftX,m_TopOffset+1,m_IFShiftX,m_TopOffset+m_SpectrumHeight-2);


    if(ValidData)
    {
        DC.SetPen(m_Pen_Spectrum_Active);
        DC.DrawLines(AWidth,m_Points+m_IFBandwidthX1-m_LeftOffset,m_LeftOffset,m_TopOffset);
    }

    PrepareDC(WindowDC);

    WindowDC.Blit(0,0,Width,Height,&DC,0,0);
    
    delete Bitmap;
}

